// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2008, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.
package com.ibm.websphere.batch.samples.tests.bds;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

//import com.ibm.jzos.ZFile;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.ByteReaderPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.FileReaderPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCCallableStatementReaderPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCReaderPattern;


public class EchoReader implements FileReaderPattern, JDBCReaderPattern, ByteReaderPattern, JDBCCallableStatementReaderPattern/*, RecordOrientedDatasetReaderPattern*/ {

	public String TableNameKey = "tablename";
	protected String tableName = "IVTSCHEMA.IVTTABLE";
	protected String echoQuery = "select * from ";
	protected String callableStatment = "{ CALL IVTSCHEMA.IVTREADPROC(?,?) }";
	protected String RecordLengthKey = "EchoReader_record_length";
	protected int recordLength = 80;
	protected byte[] buf;
	int cnt = 0;
	protected BDSFWLogger logger;
	protected Properties props;
	private String jobId;
	
	private EchoDataHolder _header = null;
	private boolean processHeader = false;
	private String PROCESS_HEADER_KEY = "PROCESS_HEADER";
	
	
	
	public Object fetchRecord(BufferedReader reader) throws IOException {	
		EchoDataHolder edc = null;
		if (logger.isDebugEnabled())
			logger.debug("Entering EchoReader.fetchRecord(bufferedReader)");
		String line = reader.readLine();
		if (line != null)
		{
			if (logger.isDebugEnabled())
				logger.debug("EchoReader.fetchRecord(bufferedReader)- line= " + line);
			edc = new EchoDataHolder(line);
		}
		else 
		{
			if (logger.isDebugEnabled())
				logger.debug("EchoReader.fetchRecord(bufferedReader)- returning null");
			
		}
		return edc;
	}

	public void initialize(Properties props) {
		
		logger = new BDSFWLogger(props);
		this.props = props;
		if (logger.isDebugEnabled())
			logger.debug("entering EchoReader.initialize()");
		
		String jobStepId = props.get(BDSFrameworkConstants.JOB_STEP_ID).toString();
		
		jobId = jobStepId.substring(0, jobStepId.indexOf('/'));
		
		String tableNameProp =props.getProperty(this.TableNameKey);
		if (tableNameProp != null)
		{
			this.tableName = tableNameProp;
		}
		// table name		
		echoQuery += tableName;
		echoQuery += " WHERE JOBID = '" + jobId + "' ORDER BY COUNT";
		
		if (props.containsKey(RecordLengthKey))
		{
			try
			{
				recordLength = Integer.valueOf(props.getProperty(RecordLengthKey).toString()).intValue();				
			}
			catch (Throwable t)
			{
				throw new RuntimeException("Invalid record length specified in EchoReader. Value= |" + props.getProperty(RecordLengthKey) + "|");
			}
		}
		buf = new byte[recordLength];
		
		String processHeaderStr = props.getProperty(PROCESS_HEADER_KEY);
		if(processHeaderStr != null && processHeaderStr.equalsIgnoreCase("true")) {
			processHeader = true;
		}
			
		
		if (logger.isDebugEnabled())
			logger.debug("exiting EchoReader.initialize()");
		
	}

	public void processHeader(BufferedReader reader) throws IOException {	
		if(processHeader) {
			String headerString = reader.readLine();
			_header = new EchoDataHolder(headerString);
		}	
	}

	public Object fetchRecord(ResultSet resultSet) {
		try 
		{
			if (logger.isDebugEnabled())
				logger.debug("entering EchoReader.fetchRecord(resultSet)");
			
			ResultSetMetaData rsmd = resultSet.getMetaData();
			int columnCount = rsmd.getColumnCount();			
						
			HashMap dbMap = new HashMap();
			for (int i = 1; i <= columnCount; i++)
			{
				String columnName = rsmd.getColumnName(i);
				
				Object columnValue = resultSet.getObject(i);
				dbMap.put(columnName, columnValue);				
			}			
			
			if (logger.isDebugEnabled())
			{
				logger.debug("EchoReader.fetchRecord(resultSet)- dbmap = " + dbMap);
				logger.debug("exiting EchoReader.fetchRecord(resultSet)");
			}
			if (dbMap.size() == 0)
				return null;
			else
			{	
				cnt++;
				return new EchoDataHolder(dbMap);
			}	
		}
		catch (Throwable t)
		{
			throw new RuntimeException (t);
		}
	}

	
	public String getRestartTokens() {
		return new Integer(cnt).toString();
	}

	public Object fetchRecord(BufferedInputStream reader) throws IOException {
		
		buf = new byte[recordLength];
		int nread = reader.read(buf);			
		
		if (logger.isDebugEnabled())
			logger.debug("fetchRecord(bufferedInputStream)\nbuf: " + buf + "\nnread=" + nread);
		
		if (nread > 0)
			return new EchoDataHolder(buf,nread);
		else
			return null;			
	}

	public void processHeader(BufferedInputStream reader) throws IOException {
		if (processHeader) {
			String header = "--HEADER--";
			byte [] bar = header.getBytes();
			byte [] buf = new byte[bar.length];
			int nread = reader.read(buf);
			if (nread > 0)
				_header = new EchoDataHolder(buf,nread);
			if (logger.isDebugEnabled())
				logger.debug("read header: " + _header);
		}	
	}
	/*public Object fetchRecord(ZFile reader) throws IOException {
		byte[] buf = new byte[reader.getLrecl()];			
		int nread = reader.read(buf);					
		
		if (logger.isDebugEnabled())
			logger.debug("fetchRecord(zfile)\nbuf: " + buf + "\nnread=" + nread);
		
		if (nread > 0)
		{
			if (logger.isDebugEnabled())
				logger.debug("nread is > 0, returning object");
			return new EchoDataHolder(buf,nread);
		}
		else
		{
			if (logger.isDebugEnabled())
				logger.debug("nread is < 0, returning null");
			return null;
		}
	}

	public void processHeader(ZFile reader) throws IOException {
		if (processHeader) {
			String header = "--HEADER--";
			byte [] bar = header.getBytes();
			byte [] buf = new byte[bar.length];
			int nread = reader.read(buf);
			if (nread > 0)
				_header = new EchoDataHolder(buf,nread);
			if (logger.isDebugEnabled())
				logger.debug("read header: " + _header);	
			
		}	
	}	*/
	
	public String getInitialLookupQuery() {
		if(logger.isDebugEnabled())
			logger.debug("Initial Query: " + echoQuery);
		return echoQuery;
	}

	
	public String getRestartQuery(String restartToken) {
		String query = "SELECT * FROM " + tableName + " WHERE COUNT > " + restartToken + " AND JOBID = '" + jobId + "' ORDER BY COUNT";
		cnt = Integer.parseInt(restartToken);
		if(logger.isDebugEnabled())
			logger.debug("Restart Query: " + query);
		return query;
	}

	public CallableStatement getInitialCallableStatement(Connection con) {
		try {
			CallableStatement cs =con.prepareCall(callableStatment);
			cs.setString(1, jobId);
			cs.setString(2, null);
			return cs;
		} catch (SQLException e) {
			e.printStackTrace();
			String msg = "SQLException caught preparing callable statement "+
			              callableStatment+": "+e.getLocalizedMessage();
			logger.error(msg);
			throw new RuntimeException(msg);
		}
	}

	public CallableStatement getRestartCallableStatement(Connection con,
			String restartToken) {
		try {
			CallableStatement cs =con.prepareCall(callableStatment);
			cnt = Integer.parseInt(restartToken);
			cs.setString(1, jobId);
			cs.setInt(2, cnt);
			return cs;
		} catch (Exception e) {
			e.printStackTrace();
			String msg = "Exception caught preparing callable statement "+
			              callableStatment+": "+e.getLocalizedMessage();
			logger.error(msg);
			throw new RuntimeException(msg);
		}
	}	
	
	public ResultSet getResultSet(CallableStatement cstmt) throws SQLException {
		return cstmt.executeQuery();
	}

	public Object fetchHeader() {
		if(processHeader) {
			return _header;
		}
		return null;
	}

	
	
}
