// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2008, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.
package com.ibm.websphere.batch.samples.tests.bds;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.IOException;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.util.Properties;

//import com.ibm.jzos.ZFile;
//import com.ibm.jzos.ZFileException;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.ByteWriterPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.FileWriterPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCCallableStatementWriterPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCWriterPattern;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.RecordOrientedDatasetWriterPattern;

public class EchoWriter implements FileWriterPattern, JDBCWriterPattern, ByteWriterPattern, JDBCCallableStatementWriterPattern/*, RecordOrientedDatasetWriterPattern*/ {

	public String TableNameKey = "tablename";
	protected String tableName = "IVTSCHEMA.IVTTABLE";
	protected String sqlQueryPreTablename = "insert into ";
	protected String sqlQueryPostTablename = " values (?, ?, ?)";
	protected String callableStatment = "{ CALL IVTSCHEMA.IVTWRITEPROC(?,?,?) }";
	protected int counter = 0;
	protected String jobid;	
	protected String jobIdKey = "JobStepId";
	public String paddedBufferKey = "padBuffer";
	public String paddedSizeKey = "bufferSize";
	boolean padBuffer = false;
	int paddedSize = 80;
	byte[] pad;
	
	protected BDSFWLogger logger;
	private boolean isRestart;
	
	public String getSQLQuery() {
		
		String sqlQuery = sqlQueryPreTablename + tableName + sqlQueryPostTablename;
		return sqlQuery;
	}

	public PreparedStatement writeRecord(PreparedStatement pstmt, Object record) {		
		
		try 
		{
			if (logger.isDebugEnabled())
				logger.debug("EchoWriter.writeRecord(PreparedStatement)- record= " + record);
			pstmt.setString(1, this.jobid);
			String rec = record.toString();
			int st = rec.indexOf(':');
			int lt = rec.lastIndexOf(':');
			String cntStr = rec.substring(st + 1, lt);
			int cnt = Integer.parseInt(cntStr);
			pstmt.setInt(2, cnt);
			
			
			pstmt.setString(3, record.toString());
			counter++;
		}
		catch (Throwable t)
		{
			
			throw new RuntimeException(t);
		}
		return pstmt;
	}

	public void initialize(Properties props) {
		logger = new BDSFWLogger(props);
		
		String jobStepId = props.get(BDSFrameworkConstants.JOB_STEP_ID).toString();
		jobid = jobStepId.substring(0, jobStepId.indexOf('/'));
		
		String jobRestartedStr = (String)props.getProperty(BDSFrameworkConstants.IS_JOB_RESTART);
		if(jobRestartedStr != null && jobRestartedStr.equals("true")) {
			
			isRestart = true;
		}
		
		
		String tableNameProp =props.getProperty(this.TableNameKey);
		if (tableNameProp != null)
		{
			this.tableName = tableNameProp;
		}
		
		String paddedBufferProp =props.getProperty(this.paddedBufferKey);
		if (paddedBufferProp != null)
		{
			if (paddedBufferProp.trim().toLowerCase().equals("true"))
				this.padBuffer = true;
			else this.padBuffer = false;
			
		}
		
		if (this.padBuffer)
		{
			String paddedBufferSizeProp =props.getProperty(this.paddedSizeKey);
			if (paddedBufferSizeProp != null)
			{
				try
				{
					this.paddedSize = Integer.valueOf(paddedBufferSizeProp).intValue();					
				}
				catch (Exception e)
				{
					logger.error("EchoWriter - invalid pad size specified |" + paddedBufferSizeProp + "| Defaulting to: " + paddedSize );
				}
			}
			pad = new byte[this.paddedSize];				
			for (int i = 0; i < this.paddedSize; i++)
				pad[i] = ' ';			
		}
	}

	public void writeHeader(BufferedWriter out) throws IOException {
	}

	public void writeRecord(BufferedWriter out, Object record)
			throws IOException {
		if (logger.isDebugEnabled())
			logger.debug("In EchoWriter:writeRecord");
		if (counter != 0 || isRestart == true)
		{
			//System.out.println("counter != 0 inserting newline" + counter);
			out.newLine();			
		}
		counter++;
		if (logger.isDebugEnabled())
			logger.debug("EchoWriter.writeRecord(BufferedWriter)- record= " + record);
		
		out.write(record.toString() );
		//out.newLine();	
		
	}

	public void writeHeader(BufferedOutputStream out) throws IOException {		
	}

	public void writeRecord(BufferedOutputStream out, Object record) throws IOException {
		
		EchoDataHolder holder = ((EchoDataHolder) record);	
		if (logger.isDebugEnabled())
			logger.debug("writeRecord(bufferedOutputStream)\nbuf: " + holder + "\nnread=" + holder.nread);
		out.write(holder.getByteData(), 0, holder.nread);
		
		if (this.padBuffer)
		{
			int pad = this.paddedSize - holder.nread;
			
			if (logger.isDebugEnabled())
				logger.debug("writeRecord(bufferedOutputStream)- padding buffer with " + pad + " spaces");
			
			if (pad > 0)
				out.write(this.pad, 0, pad-1);
		}
	}
	
	public void writeHeader(BufferedWriter out, Object header)
			throws IOException {
		EchoDataHolder edh = (EchoDataHolder)header;
		out.write(edh.toString() + System.getProperty("line.separator"));
		
		
	}
	/*public void writeRecord(ZFile out, Object record) throws IOException {			
		EchoDataHolder holder = ((EchoDataHolder) record);	
		if (logger.isDebugEnabled())
			logger.debug("writeRecord(zFile)\nbuf: " + holder + "\nnread=" + holder.nread);
		out.write(holder.getByteData(), 0, holder.nread);	
		
		if (this.padBuffer)
		{
			int pad = this.paddedSize - holder.nread;
			if (pad > 0)
				out.write(this.pad, holder.nread, this.paddedSize);
		}
	}	*/

	public void writeHeader(BufferedOutputStream out, Object header)
			throws IOException {
		//System.out.println("About to write header: " + header.toString() + header.getClass());
		EchoDataHolder holder = ((EchoDataHolder) header);	
		out.write(holder.getByteData(), 0, holder.nread);
				
	}

	public String getCallableStatementSQL() {
		
		return this.callableStatment;
	}

	public CallableStatement writeRecord(CallableStatement cstmt, Object record) {
		try 
		{
			if (logger.isDebugEnabled())
				logger.debug("EchoWriter.writeRecord(CallableStatement)- record= " + record);
			cstmt.setString(1, this.jobid);
			String rec = record.toString();
			int st = rec.indexOf(':');
			int lt = rec.lastIndexOf(':');
			String cntStr = rec.substring(st + 1, lt);
			int cnt = Integer.parseInt(cntStr);
			cstmt.setInt(2, cnt);
			
			
			cstmt.setString(3, record.toString());
			counter++;
		}
		catch (Throwable t)
		{
			
			throw new RuntimeException(t);
		}
		return cstmt;
	}

	/*public void writeHeader(ZFile out, Object header) {
		EchoDataHolder holder = ((EchoDataHolder) header);	
		try {
			out.write(holder.getByteData(), 0, holder.nread);
		} catch (ZFileException e) {
			
			e.printStackTrace();
		}
		
	}

	public void writeHeader(ZFile out) throws IOException {
		
		
	}*/

}	