// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2008, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.
/*
 * Created on May 24, 2007
 */
package com.ibm.websphere.batch.samples.tests.steps;

import java.util.Properties;

import com.ibm.websphere.batch.BatchConstants;
import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.BatchDataStreamMgr;
import com.ibm.websphere.batch.BatchJobStepInterface;
import com.ibm.websphere.batch.JobStepID;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.datastreams.bdsadapter.AbstractBatchDataInputStream;
import com.ibm.websphere.batch.samples.tests.bds.EchoDataHolder;

/**
 * @author Neeraj
 */
public class DataIntegrityVerificationStep implements BatchJobStepInterface {

	public static final int RC_OK = 0;
	public static final int RC_WARNING = 4;
	public static final int RC_ERROR = 8;
	public static final int RC_FATAL = 12;
	
    protected Properties props;
    
    protected AbstractBatchDataInputStream inputStream;
    protected AbstractBatchDataInputStream generatedOutputInputStream;
	
	protected BDSFWLogger logger;
	private long time;
	
	   
    protected void setReturnCode(int returnCode) {
    	
    }

    /**
     * Convenience method to return a property value.
     * @param key
     *   Property key.
     * @return
     *   Property value.
     */
    protected String getProperty(String key) {
        return getProperties().getProperty(key);
    }

    /**
     * Convenience method to return a property value.
     * @param key
     *   Property key.
     * @param defaultValue
     *   Default value if property was not set.
     * @return
     *   Property value.
     */
    protected String getProperty(String key, String defaultValue) {
        String value = getProperty(key);
        if (value == null) {
            value = defaultValue;
        }
        return value;
    }

    /**
     * Convenience method to return a property value.
     * Throws an exception if the property was not set, or was
     * set to an empty value.
     * 
     * @param key
     *   Property key.
     * @return
     *   Property value.
     *  
     * @throws RuntimeException
     *   No value was specified for the given key.
     */
    protected String getRequiredProperty(String key) {
        String value = getProperties().getProperty(key);
        if (value == null) {
            throw new RuntimeException("Missing required property '" + key + "'");
        }
        value = value.trim();
        if (value.length() == 0) {
            throw new RuntimeException("Required property '" + key + "' has empty value");
        }
        return value;
    }

    public void createJobStep() {
        try {            	        	
        	
        	logger = new BDSFWLogger(this.getProperties());
        	
        	if (logger.isDebugEnabled())
        		logger.debug("entering dataintegritystep:createJobStep()- jobId="+getJobID());

        	time = System.currentTimeMillis();
        	
        	inputStream = (AbstractBatchDataInputStream)BatchDataStreamMgr.getBatchDataStream("inputStream", getStepID());
            generatedOutputInputStream = (AbstractBatchDataInputStream)BatchDataStreamMgr.getBatchDataStream("generatedOutputInputStream", getStepID());
          
    		
        	if (logger.isDebugEnabled())
        		logger.debug("exiting dataintegritystep:createJobStep");

        }
        catch(Exception e) {
            throw new RuntimeException("Unexpected error in step setup", e);
        }
    }

    /**
     * Perform cleanup and pass return code back to LREE.
     */
    public int destroyJobStep() { 
    	
    	if (logger.isDebugEnabled())
    		logger.debug("entering dataintegritystep:destroyJobStep");
    	            
        logger.info("dataintegritystep.destroyStep()- Total Execution Time: " + (System.currentTimeMillis() - time));
        
    	if (logger.isDebugEnabled())
    		logger.debug( "destroyJobStep");

        return 0;
    }

    public Properties getProperties() {
        return props;
    }

    public int processJobStep() {    	    	
    	
    	if (logger.isDebugEnabled())
    		logger.debug("dataintegritystep:processJobStep");

    	try
    	{    		
    		    		
    		if (inputStream == null) {
    			logger.error("No Inputstream defined for the batch step!");
    			return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
    		}	
    		
    		if (generatedOutputInputStream == null) {
    			logger.error("No Outputstream defined for the batch step!");
    			return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
    		}	
    		
    		boolean isHasNext = inputStream.hasNext() ;
    		boolean osHasNext = generatedOutputInputStream.hasNext();
    		
    		// one stream has data while another does not thats an error
            if ( (isHasNext && !(osHasNext)) || (!(isHasNext) && osHasNext) ) 
            {
            	return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
            	
            }
            else if(isHasNext && osHasNext) // there is data to process
            {	
             	if (logger.isDebugEnabled())
	            		logger.debug("Inputstream has data to process.");
	
                EchoDataHolder iRecord = (EchoDataHolder)inputStream.read(); 
                
                EchoDataHolder oRecord = (EchoDataHolder)generatedOutputInputStream.read();
            	
                   			
                if(!iRecord.getStringData().equals(oRecord.getStringData())) 
                {
                	// data in the 2 streams not equal
                	logger.error("Input and output files are not identical" );
                	logger.error(iRecord.getStringData() + "!=" +  oRecord.getStringData() );
                	return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
                }
                	
                return BatchConstants.STEP_CONTINUE;
            	
            	
            }
            else 
            { // reached the end of both streams
            
            	return BatchConstants.STEP_COMPLETE;   
                    
            }    
                
            
        }
    	catch (BatchContainerDataStreamException e)
    	{            	
    		throw new RuntimeException("failed to obtain batch input/output streams", e);
    	}        
        catch(Exception e) {
            throw new RuntimeException("Unexcepted error in batch loop", e);
        }
    }

   

	
    /**
     * Called by LREE to set our properties as specified in xJCL.
     */
    public void setProperties(Properties props) {
    	
        this.props = props;
        props.put("JobStepId", getJobID());
    }

    // TODO: protected abstract String _getJobID();
    public String getJobID() {
        return  (String) getProperties().get(BatchConstants.JOB_ID); //JobStepContextMgr.getContext().getJobID();// return Integer.toString(hashCode());

    }
    

    public String getStepID() {
    	String jobID = (String) getProperties().get(BatchConstants.JOB_ID);
        String stepID = (String) getProperties().get(BatchConstants.STEP_ID);

       	// the bds logical name and the jobid/stepid are needed to retrieve the bds. 
        JobStepID jsId = new JobStepID( jobID, stepID );
        
        return  jsId.getJobstepid(); //JobStepContextMgr.getContext().getJobID();// return Integer.toString(hashCode());

    }


  
  
}
