// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2008, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.
/*
 * Created on May 24, 2007
 */
package com.ibm.websphere.batch.samples.tests.steps;
import java.io.BufferedWriter;
import java.io.Externalizable;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Map;
import java.util.Properties;
import java.util.Random;

import com.ibm.websphere.batch.BatchConstants;
import com.ibm.websphere.batch.context.JobStepContext;
import com.ibm.websphere.batch.context.JobStepContextMgr;
import com.ibm.websphere.batch.context.PersistentMap;
import com.ibm.websphere.batch.context.PersistentMapFactory;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.samples.tests.bds.EchoDataHolder;
import com.ibm.websphere.ci.CIWork;


public class GenerateDataStep implements CIWork {

		
    protected Properties props = new Properties();
    
    private static final String GENERATE_HEADER_KEY = "GENERATE_HEADER";
    protected String fileName;
	
	 // random functions for generating account numbers and credit/debit information 
    private Random creditdebit = new Random();
    private Random account = new Random();
    private Random amount = new Random();
    private long records = 0;
    
    private static int accountNumberHigh  =  9999;
    private static int accountNumberLow   =  1000;
    private static int accountNumberRange =  accountNumberHigh-accountNumberLow;


    private String fileEncoding = null;
    
    private boolean genHeader = false;
   
    // the default number of Postings to create
    private static String DEFAULT_RECORD_COUNT = "100";
    private long maxRecords = 100;

    // the default percentage of postings that should be debits
    private static final String DEFAULT_DEBIT_PERCENT = "2";
    private int debitPercent = new Integer(DEFAULT_DEBIT_PERCENT).intValue();

    // the default total number of jobs which may be concurrently executing
    private static final String DEFAULT_CONCURRENT_JOB_NUMBER = "1";
    private int concurrentJobs = new Integer(DEFAULT_CONCURRENT_JOB_NUMBER).intValue();

    // the default relative job number of all jobs which may be concurrently executing
    private static final String DEFAULT_RELATIVE_JOB_NUMBER = "1";
	private static final int CREDIT_TRAN_KEY = 1;
	private static final int DEBIT_TRAN_KEY = 0;
    private int relativeJob = new Integer(DEFAULT_RELATIVE_JOB_NUMBER).intValue();
   	
    private BufferedWriter _writer;
	protected BDSFWLogger logger;
	private boolean abort;
	
	
	
	public Map getProperties() {
		
		return props;
	}

	
	public void setProperties(Map prop) {
		this.props.putAll(prop);
		_init();
	}


	public boolean isDaemon() {
		// TODO Auto-generated method stub
		return false;
	}


	public void release() {
		abort = true;

	}


	public void run() {
		RuntimeException ex = null;
		System.out.println("logger: " + logger);
		if (logger.isDebugEnabled())
    		logger.debug("entering GenerateDataCIStep:run- jobId="+getJobID());
		
		boolean firstRecord = true;
		try {
			_openWriter();
			
			if(genHeader) {
				
	        		String header = "--HEADER--";
	        		_writer.write(header);
	        		_writer.newLine();
	        		
				
			}
			Object record = _generateRecord();
			while(record != null && !abort) {
				if(!firstRecord) {
					_writer.newLine();
					
				}	
				firstRecord = false;
				
				if(record != null) {
					_writer.write(record.toString());
					
				
				}
				record = _generateRecord();
			} 
			_closeWriter();

			JobStepContext context = JobStepContextMgr.getContext();
			System.out.println("getStepLevelTransientUserData()=" + (String) context.getStepLevelTransientUserData());
			System.out.println("getJobLevelTransientUserData()=" + (String) context.getJobLevelTransientUserData());

		if (logger.isDebugEnabled())
    		logger.debug("exiting GenerateDataCIStep:run- jobId="+getJobID());
			
		} catch (Throwable e) {
			
			_closeWriter();
			throw new RuntimeException(e);
		} 
		

	}
	
	 private void _closeWriter()  {
		try {
			_writer.close();
		} catch (IOException e) {
			logger.error(e.getMessage());
		}
		
	}

	private Object _generateRecord() 
	    {
	    	if (logger.isDebugEnabled())
				logger.debug("Entering GenerateDataCIStep:generateRecord");
	    	
	    	
			 // keep track of how many postings have been generated so far.
	    	records ++;
	    	Object returnValue = null;

	        if (records <= maxRecords) {
	        	
	            // Some percentage of postings are debits and some are credits. They are differentiated by static constants on the Posting class.
	            int tran = creditdebit.nextInt(101)+1 > debitPercent ? CREDIT_TRAN_KEY : DEBIT_TRAN_KEY;  

	            // Generate a random account number:
	            // The account numbers can range from 1000 to 9999.  The integers relativeJob and concurrentJobs are used
	            // to partition the account number range for this relative job such that duplicate key exceptions are avoided.
	            int accountLimit = ( accountNumberRange / concurrentJobs ) + 1;
	            int acct = account.nextInt( accountLimit ) + 1000 +  ( relativeJob-1 )*accountLimit;

	            // generate a random posting amount
	            double amt = amount.nextInt((acct/10)+1);                      // random posting amount

	            int actNo = (int)records;;
	            // create a Posting object and write it to the output stream
	            EchoDataHolder p = new EchoDataHolder( new String(tran + ":" + new Integer(actNo).toString() + ":" + amt) );

	            // log a message to the job log
	            String msg = "datacreation" + "." + "processJobStep()" 
	                + "  record=" + new Long(records).longValue()
	                + "  account=" + new Integer(acct).intValue()
	                + "  amount=" + new Double(amt).doubleValue()
	                + "  tran=" + ( tran==CREDIT_TRAN_KEY ? "credit" : "debit" )
	                + " max records = " + maxRecords ;
	            if (logger.isDebugEnabled())
	            	logger.debug( getJobID() + msg );
	            if (records < maxRecords)
	            	returnValue = p;

	        }

	        // if the specified number of postings has been generated, this step is complete, otherwise processJobStep() needs to be called 
	        // again.
	        if (logger.isDebugEnabled())
	        	logger.debug("Exiting GenerateDataCIStep:generateRecord with " + returnValue);
	        return returnValue;
		}
	
	private void _openWriter() throws IOException {
		if (logger.isDebugEnabled())
    		logger.debug("entering GenerateDataCIStep:openWriter- jobId="+getJobID());
		try
		{
			
			FileOutputStream fos = new FileOutputStream(fileName);
			
						
			if (this.fileEncoding != null) 	{
				_writer = new BufferedWriter(new OutputStreamWriter(fos, this.fileEncoding));
				
			} else 	{	
				_writer = new BufferedWriter(new OutputStreamWriter(fos));
			}			
		} catch (IOException e) {
			System.err.println("Error opening file: " + fileName);
			throw e;
			 
		}
		if (logger.isDebugEnabled())
    		logger.debug("exiting GenerateDataCIStep:openWriter- jobId="+getJobID());
	}
	
	 /**
     * Convenience method to return a property value.
     * @param key
     *   Property key.
     * @return
     *   Property value.
     */
    protected String getProperty(String key) {
        return props.getProperty(key);
    }

    /**
     * Convenience method to return a property value.
     * @param key
     *   Property key.
     * @param defaultValue
     *   Default value if property was not set.
     * @return
     *   Property value.
     */
    protected String getProperty(String key, String defaultValue) {
        String value = getProperty(key);
        if (value == null) {
            value = defaultValue;
        }
        return value;
    }

    /**
     * Convenience method to return a property value.
     * Throws an exception if the property was not set, or was
     * set to an empty value.
     * 
     * @param key
     *   Property key.
     * @return
     *   Property value.
     *  
     * @throws RuntimeException
     *   No value was specified for the given key.
     */
    protected String getRequiredProperty(String key) {
        String value = props.getProperty(key);
        if (value == null) {
            throw new RuntimeException("Missing required property '" + key + "'");
        }
        value = value.trim();
        if (value.length() == 0) {
            throw new RuntimeException("Required property '" + key + "' has empty value");
        }
        return value;
    }
	
	private void _init() {
		try {            	        	
        	
        	logger = new BDSFWLogger(props);
        	
        	if (logger.isDebugEnabled())
        		logger.debug("entering GenerateDataCIStep:init- jobId="+getJobID());

        
        	String recCnt = getProperty("RECORD_COUNT");
        	if(recCnt != null)		
        	{	
        		DEFAULT_RECORD_COUNT = recCnt;
        		if (logger.isDebugEnabled())
            		logger.debug("Setting maxrecords to "+ recCnt);

        		
        	}	
        	
        	maxRecords = new Long(DEFAULT_RECORD_COUNT).longValue();
            
            fileEncoding = getProperty(BDSFrameworkConstants.FILE_ENCODING_KEY);
            
            fileName = getRequiredProperty(BDSFrameworkConstants.FILENAME_KEY);
            
            String generateHeader = getProperty(GENERATE_HEADER_KEY);
        	//System.out.println("generateHeader " + generateHeader);
        	if(generateHeader != null && generateHeader.equalsIgnoreCase("true")) {
        		
	        	genHeader = true;
        	}	
        	
        	String appendJobIdProp = getProperty(BDSFrameworkConstants.APPEND_JOBID_TO_FILE_NAME_KEY);
    		if (appendJobIdProp == null || appendJobIdProp.equalsIgnoreCase("true"))
    		{						
    			String jobStepId = getProperty(BDSFrameworkConstants.JOB_STEP_ID).toString();
    			
    			String jobId = jobStepId.substring(0, jobStepId.indexOf('/'));
    			
    			jobId = jobId.replace('/', '.');
    			jobId = jobId.replace(':', '.');
    			jobId = jobId.replace('$', '.');
    			fileName += "." + jobId;				
    			
    		}
        	if (logger.isDebugEnabled())
        		logger.debug("exiting GenerateDataCIStep:init");
    		JobStepContext context = JobStepContextMgr.getContext();
    		com.ibm.websphere.batch.context.PersistentMap map = PersistentMapFactory.createPersistentMap();
    		map.setString("key1", "value1");
    		map.setInt("key2", 2);
    		//context.setStepLevelPersistentData(map);
    		context.setJobLevelPersistentUserData(map);
    		context.setJobLevelTransientUserData("ivt job level transient data");
    		context.setStepLevelTransientUserData("ivt step level transient data");

        }
        catch(Exception e) {
            throw new RuntimeException("Unexpected error in step setup", e);
        }
	}

	
	private String getJobID() {
		return  (String) props.get(BatchConstants.JOB_ID); 
	}

}
